//+=============================================================================
//
// file :         Fug.cpp
//
// description :  C++ source for the Fug and its commands.
//                The class is derived from Device. It represents the
//                CORBA servant object which will be accessed from the
//                network. All commands which can be executed on the
//                Fug are implemented in this file.
//
// project :      TANGO Device Server
//
// $Author: giulio $
//
// $Revision: 1.1.1.1 $
//
// $Log: Fug.cpp,v $
// Revision 1.1.1.1  2014-10-22 10:12:39  giulio
// First cvs release
//
//
// copyleft :   Sincrotrone Trieste S.C.p.A.
//              Strada Statale 14 - km 163,5 in AREA Science Park
//              34012 Basovizza (TS)
//              ITALY
//
//-=============================================================================
//
//  		This file is generated by POGO
//	(Program Obviously used to Generate tango Object)
//
//         (c) - Software Engineering Group - ESRF
//=============================================================================

//===================================================================
//
//	The following table gives the correspondance
//	between commands and method's name.
//
//  Command's name      |  Method's name
//	----------------------------------------
//  State               |  dev_state()
//  Status              |  dev_status()
//  On                  |  on()
//  Off                 |  off()
//  SetIntegrationTime  |  set_integration_time()
//  GetIntegrationTime  |  get_integration_time()
//
//===================================================================

#include <Fug.h>
#include <FugClass.h>
#include <tango.h>

#include <iostream>
#include <string>
#include <vector>

#include "fugthread.h"

namespace Fug_ns {

//+----------------------------------------------------------------------------
//
// method : 		Fug::Fug(string &s)
//
// description : 	constructor for simulated Fug
//
// in : - cl : Pointer to the DeviceClass object
//      - s : Device name
//
//-----------------------------------------------------------------------------
Fug::Fug(Tango::DeviceClass *cl, string &s) : TANGO_BASE_CLASS(cl, s.c_str()) {
  init_device();
}

Fug::Fug(Tango::DeviceClass *cl, const char *s) : TANGO_BASE_CLASS(cl, s) {
  init_device();
}

Fug::Fug(Tango::DeviceClass *cl, const char *s, const char *d)
    : TANGO_BASE_CLASS(cl, s, d) {
  init_device();
}
//+----------------------------------------------------------------------------
//
// method : 		Fug::delete_device()
//
// description : 	will be called at device destruction or at init command.
//
//-----------------------------------------------------------------------------
void Fug::delete_device() {
  //	Delete device's allocated object
  abortflag = 1;

  sleep(2);

  if (devserial) {
    delete devserial;
    devserial = 0;
  }

  if (attr_Voltage_read) {
    delete attr_Voltage_read;
    attr_Voltage_read = 0;
  }

  if (attr_Current_read) {
    delete attr_Current_read;
    attr_Current_read = 0;
  }

  if (attr_VoltageSet_read) {
    delete attr_VoltageSet_read;
    attr_VoltageSet_read = 0;
  }

  if (attr_CurrentSet_read) {
    delete attr_CurrentSet_read;
    attr_CurrentSet_read = 0;
  }

  if (attr_MaxDiffVoltage_read) {
    delete attr_MaxDiffVoltage_read;
    attr_MaxDiffVoltage_read = 0;
  }

  delete mutex;
}
/*
 * timestamp() 
 * send a timestamp to standard output
 */
void Fug::msg2err(const char *msg) 
{
	struct tm *mytime;
	char timeLong[80];
	time_t myTime_t;
	myTime_t = time(NULL);
	mytime = localtime(&myTime_t);
	// memset(mytime, 0, sizeof(tmTemp));
	strftime(timeLong, 79, "%Y-%m-%d %H:%M:%S ", mytime);
	// strftime(timeShort, 79, "%H:%M:%S ", mytime);
	cerr << timeLong << msg << endl;
}

//+----------------------------------------------------------------------------
//
// method : 		Fug::init_device()
//
// description : 	will be called at device initialization.
//
//-----------------------------------------------------------------------------
void Fug::init_device() {
  INFO_STREAM << "Fug::Fug() create device " << device_name << endl;

  // Initialise variables to default values
  //--------------------------------------------
  sprintf(rsn, "FugDeviceException");
  attr_Voltage_read = 0;
  attr_Current_read = 0;
  attr_VoltageSet_read = 0;
  attr_CurrentSet_read = 0;

  attr_Voltage_read = new Tango::DevDouble;
  if (attr_Voltage_read == 0) {
    Tango::Except::throw_exception(
        static_cast<const char *>("OUT OF MEMORY"),
        static_cast<const char *>("out of memory error"),
        static_cast<const char *>("Fug::init_device()"));
  }
  *attr_Voltage_read = 0;

  attr_Current_read = new Tango::DevDouble;
  if (attr_Current_read == 0) {
    Tango::Except::throw_exception(
        static_cast<const char *>("OUT OF MEMORY"),
        static_cast<const char *>("out of memory error"),
        static_cast<const char *>("Fug::init_device()"));
  }
  *attr_Current_read = 0;

  attr_VoltageSet_read = new Tango::DevDouble;
  if (attr_VoltageSet_read == 0) {
    Tango::Except::throw_exception(
        static_cast<const char *>("OUT OF MEMORY"),
        static_cast<const char *>("out of memory error"),
        static_cast<const char *>("Fug::init_device()"));
  }
  *attr_VoltageSet_read = 0;

  attr_CurrentSet_read = new Tango::DevDouble;
  if (attr_CurrentSet_read == 0) {
    Tango::Except::throw_exception(
        static_cast<const char *>("OUT OF MEMORY"),
        static_cast<const char *>("out of memory error"),
        static_cast<const char *>("Fug::init_device()"));
  }
  *attr_CurrentSet_read = 0;

  attr_MaxDiffVoltage_read = new Tango::DevDouble;
  if (attr_MaxDiffVoltage_read == 0) {
    Tango::Except::throw_exception(
        static_cast<const char *>("OUT OF MEMORY"),
        static_cast<const char *>("out of memory error"),
        static_cast<const char *>("Fug::init_device()"));
  }
  *attr_MaxDiffVoltage_read = 0;

  get_device_property();

  /* set exception reason */

  /* connect to serialdevice */
  devserial = new Tango::DeviceProxy(serialdevice);

  /* check if serialdevice is responding */
  try {
    if (devserial->ping()) {
      devserial->set_timeout_millis(FUG_SER_TOUT);
      set_state(Tango::ON);
      set_status("Connected to serialdevice " + serialdevice);
      cout << "Connected to serialdevice " + serialdevice << endl;
    }
  } catch (Tango::DevFailed &e) {
    set_state(Tango::FAULT);
    set_status("Failed to connect to serialdevice " + serialdevice);
    cout << "Failed to connect to serialdevice " + serialdevice << endl;
  }

  // don't readback from database
  tmpcurset = 0;
  tmpvoltset = 0;

  voltage_t.data = 0.0;
  sprintf(voltage_t.msg, "Data too old");
  voltage_t.time.tv_sec = 0;
  voltage_t.time.tv_usec = 0;

  current_t.data = 0.0;
  sprintf(current_t.msg, "Data too old");
  current_t.time.tv_sec = 0;
  current_t.time.tv_usec = 0;

  voltageset_t.data = 0.0;
  sprintf(voltageset_t.msg, "Data too old");
  voltageset_t.time.tv_sec = 0;
  voltageset_t.time.tv_usec = 0;

  currentset_t.data = 0.0;
  sprintf(current_t.msg, "Data too old");
  currentset_t.time.tv_sec = 0;
  currentset_t.time.tv_usec = 0;

  stat_t.data = 0.0;  // not useful
  sprintf(stat_t.msg, "Data too old");
  stat_t.time.tv_sec = 0;
  stat_t.time.tv_usec = 0;

  mutex = new omni_mutex();
  abortflag = 0;
  readloop = new fugthread(this);
  readloop->start();

  // millisec
  timeout =
      (5 * (((FUG_SER_TOUT + sleeptime + FUG_SER_SLEEP) * 3) + pausetime) +
       2000);
  integration_time = 100;

  for (int i = 0; i < FUG_MAX_DIFF_BUFFER; i++) {
    fug_buf[i].set_voltage = 0;
    fug_buf[i].read_voltage = 0;
    gettimeofday(&fug_buf[i].fug_time, NULL);
    fug_buf[i].fug_time.tv_sec -= 100;
    fug_buf[i].init_flag = false;
  }
  fug_buf_idx = 0;
}

//+----------------------------------------------------------------------------
//
// method : 		Fug::get_device_property()
//
// description : 	Read the device properties from database.
//
//-----------------------------------------------------------------------------
void Fug::get_device_property() {
  //	Initialize your default values here (if not done with  POGO).
  //------------------------------------------------------------------

  //	Read device properties from database.(Automatic code generation)
  //------------------------------------------------------------------
  Tango::DbData dev_prop;
  dev_prop.push_back(Tango::DbDatum("Serialdevice"));
  dev_prop.push_back(Tango::DbDatum("Maxvoltage"));
  dev_prop.push_back(Tango::DbDatum("Maxcurrent"));
  dev_prop.push_back(Tango::DbDatum("Sleeptime"));
  dev_prop.push_back(Tango::DbDatum("Readback"));
  dev_prop.push_back(Tango::DbDatum("Pausetime"));
  dev_prop.push_back(Tango::DbDatum("StringTerminator"));
  dev_prop.push_back(Tango::DbDatum("MaxDiffLen"));

  //	Call database and extract values
  //--------------------------------------------
  if (Tango::Util::instance()->_UseDb == true)
    get_db_device()->get_property(dev_prop);
  Tango::DbDatum def_prop, cl_prop;
  FugClass *ds_class = (static_cast<FugClass *>(get_device_class()));
  int i = -1;

  //	Try to initialize Serialdevice from class property
  cl_prop = ds_class->get_class_property(dev_prop[++i].name);
  if (cl_prop.is_empty() == false) cl_prop >> serialdevice;
  //	Try to initialize Serialdevice from default device value
  def_prop = ds_class->get_default_device_property(dev_prop[i].name);
  if (def_prop.is_empty() == false) def_prop >> serialdevice;
  //	And try to extract Serialdevice value from database
  if (dev_prop[i].is_empty() == false) dev_prop[i] >> serialdevice;

  //	Try to initialize Maxvoltage from class property
  cl_prop = ds_class->get_class_property(dev_prop[++i].name);
  if (cl_prop.is_empty() == false) cl_prop >> maxvoltage;
  //	Try to initialize Maxvoltage from default device value
  def_prop = ds_class->get_default_device_property(dev_prop[i].name);
  if (def_prop.is_empty() == false) def_prop >> maxvoltage;
  //	And try to extract Maxvoltage value from database
  if (dev_prop[i].is_empty() == false) dev_prop[i] >> maxvoltage;

  //	Try to initialize Maxcurrent from class property
  cl_prop = ds_class->get_class_property(dev_prop[++i].name);
  if (cl_prop.is_empty() == false) cl_prop >> maxcurrent;
  //	Try to initialize Maxcurrent from default device value
  def_prop = ds_class->get_default_device_property(dev_prop[i].name);
  if (def_prop.is_empty() == false) def_prop >> maxcurrent;
  //	And try to extract Maxcurrent value from database
  if (dev_prop[i].is_empty() == false) dev_prop[i] >> maxcurrent;

  //	Try to initialize Sleeptime from class property
  cl_prop = ds_class->get_class_property(dev_prop[++i].name);
  if (cl_prop.is_empty() == false) cl_prop >> sleeptime;
  //	Try to initialize Sleeptime from default device value
  def_prop = ds_class->get_default_device_property(dev_prop[i].name);
  if (def_prop.is_empty() == false) def_prop >> sleeptime;
  //	And try to extract Sleeptime value from database
  if (dev_prop[i].is_empty() == false) dev_prop[i] >> sleeptime;

  //	Try to initialize Readback from class property
  cl_prop = ds_class->get_class_property(dev_prop[++i].name);
  if (cl_prop.is_empty() == false) cl_prop >> readback;
  //	Try to initialize Readback from default device value
  def_prop = ds_class->get_default_device_property(dev_prop[i].name);
  if (def_prop.is_empty() == false) def_prop >> readback;
  //	And try to extract Readback value from database
  if (dev_prop[i].is_empty() == false) dev_prop[i] >> readback;

  //	Try to initialize Pausetime from class property
  cl_prop = ds_class->get_class_property(dev_prop[++i].name);
  if (cl_prop.is_empty() == false) cl_prop >> pausetime;
  //	Try to initialize Pausetime from default device value
  def_prop = ds_class->get_default_device_property(dev_prop[i].name);
  if (def_prop.is_empty() == false) def_prop >> pausetime;
  //	And try to extract Pausetime value from database
  if (dev_prop[i].is_empty() == false) dev_prop[i] >> pausetime;

  //	Try to initialize StringTerminator from class property
  cl_prop = ds_class->get_class_property(dev_prop[++i].name);
  if (cl_prop.is_empty() == false) cl_prop >> stringTerminator;
  //	Try to initialize StringTerminator from default device value
  def_prop = ds_class->get_default_device_property(dev_prop[i].name);
  if (def_prop.is_empty() == false) def_prop >> stringTerminator;
  //	And try to extract StringTerminator value from database
  if (dev_prop[i].is_empty() == false) dev_prop[i] >> stringTerminator;

  //	Try to initialize MaxDiffLen from class property
  cl_prop = ds_class->get_class_property(dev_prop[++i].name);
  if (cl_prop.is_empty() == false) cl_prop >> maxDiffLen;
  //	Try to initialize MaxDiffLen from default device value
  def_prop = ds_class->get_default_device_property(dev_prop[i].name);
  if (def_prop.is_empty() == false) def_prop >> maxDiffLen;
  //	And try to extract MaxDiffLen value from database
  if (dev_prop[i].is_empty() == false) dev_prop[i] >> maxDiffLen;

  //	End of Automatic code generation
  //------------------------------------------------------------------
}
//+----------------------------------------------------------------------------
//
// method : 		Fug::always_executed_hook()
//
// description : 	method always executed before any command is executed
//
//-----------------------------------------------------------------------------
void Fug::always_executed_hook() {}
//+----------------------------------------------------------------------------
//
// method : 		Fug::read_attr_hardware
//
// description : 	Hardware acquisition for attributes.
//
//-----------------------------------------------------------------------------
void Fug::read_attr_hardware(vector<long> &attr_list) {
  DEBUG_STREAM
      << "Fug::read_attr_hardware(vector<long> &attr_list) entering... "
      << endl;
  //	Add your own code here
}
//+----------------------------------------------------------------------------
//
// method : 		Fug::read_MaxDiffVoltage
//
// description : 	Extract real attribute values for MaxDiffVoltage
// acquisition result.
//
//-----------------------------------------------------------------------------
void Fug::read_MaxDiffVoltage(Tango::Attribute &attr) {
  DEBUG_STREAM
      << "Fug::read_MaxDiffVoltage(Tango::Attribute &attr) entering... "
      << endl;

  if (*attr_MaxDiffVoltage_read < -1*MAX_RETRY) {
    Tango::Except::throw_exception(
        static_cast<const char *>("Error reading max diff voltage"),
        static_cast<const char *>("Error reading max voltage"),
        static_cast<const char *>("Fug::read_MaxDiffVoltage"), Tango::ERR);
  }

  attr.set_value(attr_MaxDiffVoltage_read);
}

//+----------------------------------------------------------------------------
//
// method : 		Fug::read_Voltage
//
// description : 	Extract real attribute values for Voltage acquisition
// result.
//
//-----------------------------------------------------------------------------
void Fug::read_Voltage(Tango::Attribute &attr) {
  DEBUG_STREAM << "Fug::read_Voltage(Tango::Attribute &attr) entering... "
               << endl;

  struct timeval now;
  gettimeofday(&now, NULL);

  if (check_timeout(now, voltage_t.time, timeout) < 0) {
    if (voltage_t.msg[0]) {
      Tango::Except::throw_exception(
          static_cast<const char *>("Error reading voltage"),
          static_cast<const char *>(voltage_t.msg),
          static_cast<const char *>("Fug::read_Voltage"), Tango::ERR);
    } else {
      Tango::Except::throw_exception(
          static_cast<const char *>("Data is not refreshed"),
          static_cast<const char *>("Data too old"),
          static_cast<const char *>("Fug::read_Voltage"), Tango::ERR);
    }
  }
  *attr_Voltage_read = voltage_t.data;
  attr.set_value(attr_Voltage_read);
}

//+----------------------------------------------------------------------------
//
// method : 		Fug::write_Voltage
//
// description : 	Write Voltage attribute values to hardware.
//
//-----------------------------------------------------------------------------
void Fug::write_Voltage(Tango::WAttribute &attr) {
  DEBUG_STREAM << "Fug::write_Voltage(Tango::WAttribute &attr) entering... "
               << endl;

  string strout;

  attr.get_write_value(attr_Voltage_write);

  INFO_STREAM << "Fug::write_Voltage(): val = " << attr_Voltage_write << endl;

  if (attr_Voltage_write > maxvoltage) {
    Tango::Except::throw_exception(
        static_cast<const char *>("Check voltage value error"),
        static_cast<const char *>(
            "Voltage value > maxvoltage defined in device properties"),
        static_cast<const char *>("Fug::write_Voltage"), Tango::ERR);
  }

  ostringstream stream;
  stream << "U" << (attr_Voltage_write * 1000);
  string strin(stream.str());

  try {
    mutex->lock();
    write_Fug(strin);
  } catch (Tango::DevFailed &e) {
    mutex->unlock();
    Tango::Except::re_throw_exception(
        e, static_cast<const char *>("Settting voltage failed"),
        static_cast<const char *>("Error setting voltage"),
        static_cast<const char *>("Fug::write_Voltage"), Tango::ERR);
  }

  mutex->unlock();

  // store voltage setting into device property
  if (readback == false) {
    tmpvoltset = attr_Voltage_write;
  }
}

//+----------------------------------------------------------------------------
//
// method : 		Fug::read_Current
//
// description : 	Extract real attribute values for Current acquisition
// result.
//
//-----------------------------------------------------------------------------
void Fug::read_Current(Tango::Attribute &attr) {
  DEBUG_STREAM << "Fug::read_Current(Tango::Attribute &attr) entering... "
               << endl;

  struct timeval now;
  gettimeofday(&now, NULL);

  if (check_timeout(now, current_t.time, timeout) < 0) {
    if (current_t.msg[0]) {
      Tango::Except::throw_exception(
          static_cast<const char *>("Error reading current"),
          static_cast<const char *>(current_t.msg),
          static_cast<const char *>("Fug::read_Current"), Tango::ERR);
    } else {
      Tango::Except::throw_exception(
          static_cast<const char *>("Data is not refreshed"),
          static_cast<const char *>("Data too old"),
          static_cast<const char *>("Fug::read_Current"), Tango::ERR);
    }
  }
  *attr_Current_read = current_t.data;
  attr.set_value(attr_Current_read);
}

//+----------------------------------------------------------------------------
//
// method : 		Fug::write_Current
//
// description : 	Write Current attribute values to hardware.
//
//-----------------------------------------------------------------------------
void Fug::write_Current(Tango::WAttribute &attr) {
  DEBUG_STREAM << "Fug::write_Current(Tango::WAttribute &attr) entering... "
               << endl;

  string strout;

  attr.get_write_value(attr_Current_write);

  INFO_STREAM << "Fug::write_Current(): val = " << attr_Current_write << endl;

  if (attr_Current_write > maxcurrent) {
    Tango::Except::throw_exception(
        static_cast<const char *>("Check current value error"),
        static_cast<const char *>(
            "Current value > maxcurrent defined in device properties"),
        static_cast<const char *>("Fug::write_Current"), Tango::ERR);
  }
  ostringstream stream;
  stream << "I" << (attr_Current_write);
  string strin(stream.str());

  try {
    mutex->lock();
    write_Fug(strin);
  } catch (Tango::DevFailed &e) {
    mutex->unlock();
    Tango::Except::re_throw_exception(
        e, static_cast<const char *>("Setting current failed"),
        static_cast<const char *>("Error setting current"),
        static_cast<const char *>("Fug::write_Current"), Tango::ERR);
  }

  mutex->unlock();

  if (readback == false) {
    tmpcurset = attr_Current_write;
  }
}

//+----------------------------------------------------------------------------
//
// method : 		Fug::read_VoltageSet
//
// description : 	Extract real attribute values for VoltageSet acquisition
// result.
//
//-----------------------------------------------------------------------------
void Fug::read_VoltageSet(Tango::Attribute &attr) {
  DEBUG_STREAM << "Fug::read_VoltageSet(Tango::Attribute &attr) entering... "
               << endl;

  struct timeval now;
  gettimeofday(&now, NULL);

  if (check_timeout(now, voltageset_t.time, timeout) < 0) {
    if (voltageset_t.msg[0]) {
      Tango::Except::throw_exception(
          static_cast<const char *>("Error reading voltage setpoint"),
          static_cast<const char *>(voltageset_t.msg),
          static_cast<const char *>("Fug::read_VoltageSet"), Tango::ERR);
    } else {
      Tango::Except::throw_exception(
          static_cast<const char *>("Data is not refreshed"),
          static_cast<const char *>("Data too old"),
          static_cast<const char *>("Fug::read_VoltageSet"), Tango::ERR);
    }
  }
  *attr_VoltageSet_read = voltageset_t.data;
  attr.set_value(attr_VoltageSet_read);
}

//+----------------------------------------------------------------------------
//
// method : 		Fug::read_CurrentSet
//
// description : 	Extract real attribute values for CurrentSet acquisition
// result.
//
//-----------------------------------------------------------------------------
void Fug::read_CurrentSet(Tango::Attribute &attr) {
  DEBUG_STREAM << "Fug::read_CurrentSet(Tango::Attribute &attr) entering... "
               << endl;

  struct timeval now;
  gettimeofday(&now, NULL);

  if (check_timeout(now, currentset_t.time, timeout) < 0) {
    if (currentset_t.msg[0]) {
      Tango::Except::throw_exception(
          static_cast<const char *>("Error reading current setpoint"),
          static_cast<const char *>(currentset_t.msg),
          static_cast<const char *>("Fug::read_CurrentSet"), Tango::ERR);
    } else {
      Tango::Except::throw_exception(
          static_cast<const char *>("Data is not refreshed"),
          static_cast<const char *>("Data too old"),
          static_cast<const char *>("Fug::read_CurrentSet"), Tango::ERR);
    }
  }
  *attr_CurrentSet_read = currentset_t.data;
  attr.set_value(attr_CurrentSet_read);
}

//+----------------------------------------------------------------------------
//
// method : 		Fug::read_Stat
//
// description : 	Extract real attribute values for Stat acquisition
// result.
//
//-----------------------------------------------------------------------------
void Fug::read_Stat(Tango::Attribute &attr) {
  DEBUG_STREAM << "Fug::read_Stat(Tango::Attribute &attr) entering... " << endl;

  struct timeval now;
  gettimeofday(&now, NULL);

  if (check_timeout(now, stat_t.time, timeout) < 0) {
    if (stat_t.msg[0]) {
      Tango::Except::throw_exception(
          static_cast<const char *>("Error reading fug status"),
          static_cast<const char *>(stat_t.msg),
          static_cast<const char *>("Fug::read_Stat"), Tango::ERR);
    } else {
      Tango::Except::throw_exception(
          static_cast<const char *>("Data is not refreshed"),
          static_cast<const char *>("Data too old"),
          static_cast<const char *>("Fug::read_Stat"), Tango::ERR);
    }
  }
  // attr_Stat_read points to FugStat[0]
  attr_Stat_read = &FugStat[0];
  attr.set_value(attr_Stat_read, 5, 0, false);
}

//+------------------------------------------------------------------
/**
 *	method:	Fug::on
 *
 *	description:	method to execute "On"
 *	Set Power Supply Unit ON.
 *	In this particular fug model, On command isn't supported by serialline.
 *
 *
 */
//+------------------------------------------------------------------
void Fug::on() {
  DEBUG_STREAM << "Fug::on(): entering... !" << endl;

  //	Add your own code to control device here
  INFO_STREAM << "Fug::on()" << endl;
  write_Fug("F1");
}

//+------------------------------------------------------------------
/**
 *	method:	Fug::off
 *
 *	description:	method to execute "Off"
 *	Set Power Supply Unit OFF.
 *	In this particular fug model, Off command isn't supported by serialline.
 *
 *
 */
//+------------------------------------------------------------------
void Fug::off() {
  DEBUG_STREAM << "Fug::off(): entering... !" << endl;

  //	Add your own code to control device here
  INFO_STREAM << "Fug::off()" << endl;
  if (readback == false) {
    tmpcurset = 0;
    tmpvoltset = 0;
  }
  write_Fug("F0");
}

//+------------------------------------------------------------------
/**
 *	method:	Fug::set_integration_time
 *
 *	description:	method to execute "SetIntegrationTime"
 *
 * @param	argin
 *
 */
//+------------------------------------------------------------------
void Fug::set_integration_time(Tango::DevLong argin) {
  DEBUG_STREAM << "Fug::set_integration_time(): entering... !" << endl;

  //	Add your own code to control device here
  string strout;
  int intgr_time;

  INFO_STREAM << "Fug::set_integration_time(): val = " << argin << endl;

  switch (argin) {
    case 2:
      intgr_time = 0;
      break;
    case 16:
      intgr_time = 1;
      break;
    case 20:
      intgr_time = 2;
      break;
    case 50:
      intgr_time = 3;
      break;
    case 60:
      intgr_time = 4;
      break;
    case 100:
      intgr_time = 5;
      break;
    case 200:
      intgr_time = 6;
      break;
    case 800:
      intgr_time = 7;
      break;
    default:
      Tango::Except::throw_exception(
          static_cast<const char *>("Failed to set integration measure time"),
          static_cast<const char *>(
              "Allowed values [ms] are: 2, 16, 20, 50, 60, 100, 200, 800"),
          static_cast<const char *>("Fug::set_integration_time"), Tango::ERR);
  }

  ostringstream stream;
  stream << "S" << intgr_time;
  string strin(stream.str());

  try {
    mutex->lock();
    write_Fug(strin);
  } catch (Tango::DevFailed &e) {
    mutex->unlock();
    Tango::Except::re_throw_exception(
        e, static_cast<const char *>("Failed to set integration measure time"),
        static_cast<const char *>("Error getting integration time"),
        static_cast<const char *>("Fug::set_integration_time"), Tango::ERR);
  }

  integration_time = argin;

  mutex->unlock();
}

//+------------------------------------------------------------------
/**
 *	method:	Fug::get_integration_time
 *
 *	description:	method to execute "GetIntegrationTime"
 *
 * @return
 *
 */
//+------------------------------------------------------------------
Tango::DevLong Fug::get_integration_time() {
  Tango::DevLong argout;
  DEBUG_STREAM << "Fug::get_integration_time(): entering... !" << endl;

  //	Add your own code to control device here
  string strout;
  argout = integration_time;
  return (argout);
}

//+------------------------------------------------------------------
/**
 *	method:	Fug::write_Fug
 *
 *	description:	method to execute "write_Fug"
 *	Write a string to Fug serial interface
 *
 *
 */
//+------------------------------------------------------------------
void Fug::write_Fug(string input) {
  DEBUG_STREAM << "Fug::write_Fug(): entering... !" << endl;

  //	Add your own code to control device here
  Tango::DeviceData Din;

  input += stringTerminator; /* add fug endline */
  Din << input;
  try {
    devserial->command_inout("DevSerWriteString", Din);
  } catch (Tango::DevFailed &e) {
    TangoSys_MemStream out_stream;
    out_stream << "Write command to SerialDeviceServer failed" << ends;
    set_state(Tango::UNKNOWN);
    set_status("Errors from serialdevice " + serialdevice);
    Tango::Except::re_throw_exception(
        e, static_cast<const char *>(rsn), out_stream.str(),
        static_cast<const char *>("SerialDeviceServer error"), Tango::ERR);
  }
  catch (...) {
    std::exception_ptr p = std::current_exception();
    std::cerr <<(p ? p.__cxa_exception_type()->name() : "null") << std::endl;
    msg2err("DevSerWriteString write_Fug");
  }

  /*printf("write %s: %s\n",device_name.c_str(),input.c_str());*/

  set_state(Tango::ON);
  set_status("Connected to serialdevice " + serialdevice);

  omni_thread::sleep(
      0, FUG_SER_SLEEP *
             1000000);  // convert millisecond to nanosecond (sleep 70ms.)
}

//+------------------------------------------------------------------
/**
 *	method:	Fug::read_Fug
 *
 *	description:	method to execute "read_Fug"
 *	Write a string to Fug, read its answer serial interface
 *
 *
 */
//+------------------------------------------------------------------
string Fug::read_Fug() {
  DEBUG_STREAM << "Fug::read_Fug(): entering... !" << endl;

  //	Add your own code to control device here
  Tango::DevVarLongStringArray strin;
  string input;
  string strout;
  string strout_filt;
  Tango::DeviceData Din;
  Tango::DeviceData Dout;
  int i;

  input.resize(3);
  input[0] = '?';
  input[1] = stringTerminator;
  input[2] = 0;

  Din << input;
  try {
    devserial->command_inout("DevSerWriteString", Din);
  } catch (Tango::DevFailed &e) {
    TangoSys_MemStream out_stream;
    out_stream << "Write command to SerialDeviceServer failed" << ends;
    set_state(Tango::FAULT);
    set_status("Errors from serialdevice " + serialdevice);
    Tango::Except::re_throw_exception(
        e, static_cast<const char *>(rsn), out_stream.str(),
        static_cast<const char *>("SerialDeviceServer error"), Tango::ERR);
  }
  catch (...) {
    std::exception_ptr p = std::current_exception();
    std::cerr <<(p ? p.__cxa_exception_type()->name() : "null") << std::endl;
    msg2err("read_Fug DevSerWriteString");
  }

  omni_thread::sleep(
      0, sleeptime * 1000000);  // convert millisecond to microsecond

  try {
    Dout = devserial->command_inout("DevSerReadRaw");
  } catch (Tango::DevFailed &e) {
    TangoSys_MemStream out_stream;
    out_stream << "DevSerReadRaw command to SerialDeviceServer failed" << ends;
    set_state(Tango::FAULT);
    set_status("Errors from serialdevice " + serialdevice);
    Tango::Except::re_throw_exception(
        e, static_cast<const char *>(rsn), out_stream.str(),
        static_cast<const char *>("SerialDeviceServer error"), Tango::ERR);
  }
  catch (...) {
    std::exception_ptr p = std::current_exception();
    std::cerr <<(p ? p.__cxa_exception_type()->name() : "null") << std::endl;
    msg2err("read_Fug DevSerReadRaw");
  }
  Dout >> strout;

  if (strout.length() == 0) {
    set_state(Tango::FAULT);
    set_status("Errors from serialdevice " + serialdevice);
    Tango::Except::throw_exception(
        static_cast<const char *>("Serial communication error"),
        static_cast<const char *>("No data from serialline"),
        static_cast<const char *>("Fug::read_Fug"), Tango::ERR);
  }

  set_state(Tango::ON);
  set_status("Connected to serialdevice " + serialdevice);

  omni_thread::sleep(0, FUG_SER_SLEEP * 1000000);

  try {
    i = 0;
    while (strout[i] == 'E') {
      if (strout[i+3] == '\n' || strout[i+3] == '\r' || strout[i+3] == '\0') i++;
      i += 3;
    }
    strout_filt.resize(strout.length() - i);
    strncpy((char *)strout_filt.c_str(), (char *)strout.c_str() + i, strout.length() - i);
  }
  catch (...) {
    std::exception_ptr p = std::current_exception();
    std::cerr <<(p ? p.__cxa_exception_type()->name() : "null") << std::endl;
    msg2err("read_Fug misc");
  }

  /*printf("read %s: %s\n",device_name.c_str(),strout_filt.c_str());*/

  return (strout_filt);
}

//+------------------------------------------------------------------
/**
 *	method:	Fug::check_timeout
 *
 *	description:	method to execute "check_timeout"
 *	Return -1 if timeout [msec.] exceeded
 *
 *
 */
//+------------------------------------------------------------------
int Fug::check_timeout(timeval timenow, timeval time, unsigned int time_out) {
  if ((timenow.tv_sec - time.tv_sec) > (time_out / 1000)) {
    return (-1);
  }
  if (((timenow.tv_sec - time.tv_sec) * 1000 -
       (int)((timenow.tv_usec - time.tv_usec) / 1000.0)) > time_out) {
    return (-1);
  }
  return (0);

} /* end check_timeout() */

}  // namespace Fug_ns
