//=============================================================================
//
// file :        Fug.h
//
// description : Include for the Fug class.
//
// project :	FuG HV power supply device server
//
// $Author: giulio $
//
// $Revision: 1.1.1.1 $
//
// $Log: Fug.h,v $
// Revision 1.1.1.1  2014-10-22 10:12:39  giulio
// First cvs release
//
//
// copyleft :   Sincrotrone Trieste S.C.p.A.
//              Strada Statale 14 - km 163,5 in AREA Science Park
//              34012 Basovizza (TS)
//              ITALY
//
//-=============================================================================
//
//              This file is generated by POGO
//      (Program Obviously used to Generate tango Object)
//
//         (c) - Software Engineering Group - ESRF 
//=============================================================================
#ifndef _FUG_H
#define _FUG_H

#include <tango.h>
#include <ctime>

#define FUG_SER_TOUT 150 //ms.
#define FUG_SER_SLEEP 80 //ms.
#define FUG_MSG_LEN 80
#define MAX_RETRY 50

#define FUG_MAX_DIFF_BUFFER 1000


typedef struct {
	double data;
	char msg[FUG_MSG_LEN];
	struct timeval time;
} fug_t;

typedef struct {
	double set_voltage;
	double read_voltage;
	struct timeval fug_time;
	bool init_flag;
} fug_buf_t;




/**
 * @author	$Author: giulio $
 * @version	$Revision: 1.1.1.1 $
 */

 //	Add your own constants definitions here.
 //-----------------------------------------------


namespace Fug_ns
{
	class fugthread;
/**
 * Class Description:
 * This is FuG HV power supply device server.
 *	It supports the original version of FuG RS-232 interface: no reading of the
 *	setting.
 */

/*
 *	Device States Description:
 *  Tango::ON :       Serialline communication OK.
 *  Tango::UNKNOWN :  Serialline device server is not responding.
 */


class Fug: public TANGO_BASE_CLASS
{
public :
	//	Add your own data members here
	//-----------------------------------------


	//	Here is the Start of the automatic code generation part
	//-------------------------------------------------------------	
/**
 *	@name attributes
 *	Attributs member data.
 */
//@{
		Tango::DevDouble	*attr_Voltage_read;
		Tango::DevDouble	attr_Voltage_write;
		Tango::DevDouble	*attr_Current_read;
		Tango::DevDouble	attr_Current_write;
		Tango::DevDouble	*attr_VoltageSet_read;
		Tango::DevDouble	*attr_CurrentSet_read;
		Tango::DevBoolean	*attr_Stat_read;
		Tango::DevDouble	*attr_MaxDiffVoltage_read;
//@}

/**
 *	@name Device properties
 *	Device properties member data.
 */
//@{
/**
 *	name of the serialline device server
 */
	string	serialdevice;
/**
 *	Maximum voltage value [Volt]
 */
	Tango::DevDouble	maxvoltage;
/**
 *	Maximum current value [Ampere].
 */
	Tango::DevDouble	maxcurrent;
/**
 *	Sleeptime after any read or write action on serialline device in milliseconds.
 */
	Tango::DevLong	sleeptime;
/**
 *	It is true if Fug supports readback of the voltage and current settings,
 *	otherwise false
 */
	Tango::DevBoolean	readback;
/**
 *	Millisecond between each call in readloop thread
 */
	Tango::DevLong	pausetime;
/**
 *	Serial string terminator:
 *	13=old probus 3 fug serial controller
 *	10=new probus 5 fug serial controller
 */
	Tango::DevShort	stringTerminator;
/**
 *	
 */
	Tango::DevLong	maxDiffLen;
//@}

/**@name Constructors
 * Miscellaneous constructors */
//@{
/**
 * Constructs a newly allocated Command object.
 *
 *	@param cl	Class.
 *	@param s 	Device Name
 */
	Fug(Tango::DeviceClass *cl,string &s);
/**
 * Constructs a newly allocated Command object.
 *
 *	@param cl	Class.
 *	@param s 	Device Name
 */
	Fug(Tango::DeviceClass *cl,const char *s);
/**
 * Constructs a newly allocated Command object.
 *
 *	@param cl	Class.
 *	@param s 	Device name
 *	@param d	Device description.
 */
	Fug(Tango::DeviceClass *cl,const char *s,const char *d);
//@}

/**@name Destructor
 * Only one desctructor is defined for this class */
//@{
/**
 * The object desctructor.
 */	
	~Fug() {delete_device();};
/**
 *	will be called at device destruction or at init command.
 */
	void delete_device();
//@}

	
/**@name Miscellaneous methods */
//@{
/**
 *	Initialize the device
 */
	virtual void init_device();
/**
 *	Always executed method befor execution command method.
 */
	virtual void always_executed_hook();

//@}

/**
 * @name Fug methods prototypes
 */

//@{
/**
 *	Hardware acquisition for attributes.
 */
	virtual void read_attr_hardware(vector<long> &attr_list);
/**
 *	Extract real attribute values for Voltage acquisition result.
 */
	virtual void read_Voltage(Tango::Attribute &attr);
/**
 *	Write Voltage attribute values to hardware.
 */
	virtual void write_Voltage(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for Current acquisition result.
 */
	virtual void read_Current(Tango::Attribute &attr);
/**
 *	Write Current attribute values to hardware.
 */
	virtual void write_Current(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for VoltageSet acquisition result.
 */
	virtual void read_VoltageSet(Tango::Attribute &attr);
/**
 *	Extract real attribute values for CurrentSet acquisition result.
 */
	virtual void read_CurrentSet(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Stat acquisition result.
 */
	virtual void read_Stat(Tango::Attribute &attr);
/**
 *	Extract real attribute values for MaxDiffVoltage acquisition result.
 */
	virtual void read_MaxDiffVoltage(Tango::Attribute &attr);
/**
 *	Read/Write allowed for Voltage attribute.
 */
	virtual bool is_Voltage_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Current attribute.
 */
	virtual bool is_Current_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for VoltageSet attribute.
 */
	virtual bool is_VoltageSet_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for CurrentSet attribute.
 */
	virtual bool is_CurrentSet_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Stat attribute.
 */
	virtual bool is_Stat_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for MaxDiffVoltage attribute.
 */
	virtual bool is_MaxDiffVoltage_allowed(Tango::AttReqType type);
/**
 *	Execution allowed for On command.
 */
	virtual bool is_On_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Off command.
 */
	virtual bool is_Off_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for SetIntegrationTime command.
 */
	virtual bool is_SetIntegrationTime_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for GetIntegrationTime command.
 */
	virtual bool is_GetIntegrationTime_allowed(const CORBA::Any &any);
/**
 * Set Power Supply Unit ON.
 *	In this particular fug model, On command isn't supported by serialline.
 *	@exception DevFailed
 */
	void	on();
/**
 * Set Power Supply Unit OFF.
 *	In this particular fug model, Off command isn't supported by serialline.
 *	@exception DevFailed
 */
	void	off();
/**
 * 
 *	@param	argin	
 *	@exception DevFailed
 */
	void	set_integration_time(Tango::DevLong);
/**
 * 
 *	@return	
 *	@exception DevFailed
 */
	Tango::DevLong	get_integration_time();

/**
 *	Read the device properties from database
 */
	 void get_device_property();
//@}

	//	Here is the end of the automatic code generation part
	//-------------------------------------------------------------	

friend class fugthread;

private :
	void msg2err(const char *msg);
	Tango::DeviceProxy *devserial;
	int abortflag;  // flag used to abort loops (On,Off,Standby)	
	bool FugStat[5];	
	fug_t voltage_t;
	fug_t current_t;
	fug_t voltageset_t;
	fug_t currentset_t;
	fug_t stat_t;
	int integration_time;
	double tmpcurset;
	double tmpvoltset;
	fugthread *readloop;
	omni_mutex *mutex;
	unsigned int timeout;
	fug_buf_t fug_buf[FUG_MAX_DIFF_BUFFER];
	int fug_buf_idx;
	
protected :	
	//	Add your own data members here
	//-----------------------------------------
	void write_Fug(string);
	string read_Fug();
	int check_timeout(timeval, timeval, unsigned int);
	Tango::DevDouble attr_d_rd;	
	char rsn[25];  // Fug Exception reason	

};

}	// namespace_ns

#endif	// _FUG_H
